using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace Weigelt.Windows.Forms
{
	/// <summary>
	/// A filter for events that may be raised in rapid succession, to be used
	/// when only the last event (within a certain time window) is actually of interest.
	/// </summary>
	/// <typeparam name="TEventArgs">Type of the event arguments.</typeparam>
	/// <remarks>
	/// <para>
	/// Usage: Instantiate the generic class for a specific type of the event arguments
	/// and use the <see cref="HandleOriginalEvent"/> method as the handler for
	/// the original event.
	/// </para><para>
	/// The event <see cref="FilteredEventRaised"/> will be only be raised when
	/// no other event occurs with a certain time window (default: 150 msec).
	/// </para>
	/// </remarks>
	public class EventFilter<TEventArgs>
		where TEventArgs : EventArgs
	{
		private const int _DefaultTimeWindow=150; // 150msek ist i.d.R. OK
		private Timer _timer;
		private object _pendingSender;
		private TEventArgs _pendingEventArgs;

		private void Timer_Tick( object sender, EventArgs e )
		{
			_timer.Stop();

			EventHandler<TEventArgs> handler = this.FilteredEventRaised;
			if (handler!=null)
				handler( _pendingSender, _pendingEventArgs );

			_pendingSender = null;
			_pendingEventArgs = null;
		}

		/// <summary>
		/// Handles the original event.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <typeparamref name="TEventArgs"/> instance containing the event data.</param>
		public void HandleOriginalEvent( object sender, TEventArgs e )
		{
			_timer.Stop();
			_pendingSender = sender;
			_pendingEventArgs = e;
			_timer.Start();
		}

		/// <summary>
		/// Occurs when an event has finally made it through the filter.
		/// </summary>
		public event EventHandler<TEventArgs> FilteredEventRaised;

		/// <summary>
		/// Initializes a new instance of the <see cref="EventFilter&lt;TEventArgs&gt;"/> class.
		/// </summary>
		/// <param name="timeWindow">The time window in milliseconds (default is 150msec).</param>
		public EventFilter( int timeWindow )
		{
			_timer = new Timer();
			_timer.Interval = timeWindow;
			_timer.Tick += Timer_Tick;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="EventFilter&lt;TEventArgs&gt;"/> class.
		/// </summary>
		public EventFilter()
			:this(_DefaultTimeWindow)
		{
		}
	}
}
